@@ -15,11 +15,11 @@ int cg(MPI_Comm comm,Mat A, PC B,double *sol, double *rhs, int lsize)
 {
   double *ax,*z,*r,*p,*q,*x;
   double zero=0.0,one=1.0; 
-  double rho = one, rho_0 = one,l_rho,rho_old,pq,l_pq,beta,alpha;
+  double rho = one, rho_0 = one,rho_old,pq,beta,alpha;
   int it,mytid,i;
   Vec r_vec = 0, z_vec = 0, p_vec = 0,q_vec = 0;
   int ierr;
-  double t;
+  double t,tt;
 
   MPI_Comm_rank(comm,&mytid);
 
@@ -47,24 +47,26 @@ int cg(MPI_Comm comm,Mat A, PC B,double *sol, double *rhs, int lsize)
   /* Init: r = b-A*x0, with x0=0 makes r=b */
   for (i=0; i<lsize; i++) {
     sol[i] = zero;
-    r[i]=rhs[i];
+    r[i]=-rhs[i];
   }
 
-  t = PetscGetTime();
+  ierr = PetscGetTime(&t); CHKERRQ(ierr);
   /* Loop */
-  for (it=1; (rho/rho_0>.0000001) & (it<100); it++) {
+  for (it=1; (rho/rho_0>.0000001) & (it<3000); it++) {
+    double l_rho,l_pq,l_rr,rr;
 
     /* solve Mz = r */
-    /*printf("About to apply\n"); VecView(r_vec,0);*/
     ierr = PCApply(B,r_vec,z_vec); CHKERRQ(ierr);
-    /*printf("Done apply\n"); VecView(z_vec,0);*/
 
     /* rho = r^t z */
     rho_old = rho; l_rho = zero;
     for (i=0; i<lsize; i++) l_rho = l_rho + r[i]*z[i];
+    l_rr = zero;
+    for (i=0; i<lsize; i++) l_rr = l_rr + r[i]*r[i];
     MPI_Allreduce(&l_rho,&rho,1,MPI_DOUBLE,MPI_SUM,comm);
-    if (!mytid) printf("[it=%d] rMr: %e\n",it,rho);
-
+    MPI_Allreduce(&l_rr,&rr,1,MPI_DOUBLE,MPI_SUM,comm);
+    PetscPrintf(comm,"[ iteration %d ] rMr = %e, |r| = %e\n",it,rho,sqrt(rr));
+    
     if (it==1) {
       /* p=z */
       rho_0 = rho;
@@ -72,35 +74,32 @@ int cg(MPI_Comm comm,Mat A, PC B,double *sol, double *rhs, int lsize)
     } else {
       /* p=z+beta p */
       beta = rho/rho_old;
-      /*if (!mytid) printf("beta: %e\n",beta);*/
       for (i=0; i<lsize; i++) p[i] = z[i]+beta*p[i];
     }
-
+    
     /* q = A*p */
-    /*printf("Mvp on\n"); VecView(p_vec,0);*/
     ierr = MatMult(A,p_vec,q_vec); CHKERRQ(ierr);
-    /*printf("Mvp result\n"); VecView(q_vec,0);*/
-
     /* alpha = rho / p^t q;*/
     l_pq = zero;
     for (i=0; i<lsize; i++) l_pq = l_pq + p[i]*q[i];
     MPI_Allreduce(&l_pq,&pq,1,MPI_DOUBLE,MPI_SUM,comm);
     alpha = rho / pq;
-
     /* x = x+alpha p */
-    for (i=0; i<lsize; i++) x[i]=x[i]+alpha*p[i];
+    for (i=0; i<lsize; i++) x[i]=x[i]-alpha*p[i];
 
     /* r = r-alpha q */
     for (i=0; i<lsize; i++) r[i]=r[i]-alpha*q[i];
-
+    
   }
-  t = PetscGetTime()-t;
-  if (!mytid) printf("Total iteration time: %e\n",t);
-      ierr = VecDestroy(r_vec); CHKERRQ(ierr);
-      ierr = VecDestroy(z_vec); CHKERRQ(ierr);
+  ierr = PetscGetTime(&tt); CHKERRQ(ierr);
+  t = tt-t;
 
-      ierr = VecDestroy(p_vec); CHKERRQ(ierr);
-      ierr = VecDestroy(q_vec); CHKERRQ(ierr);
+  PetscPrintf(comm,"Total iteration time: %e\n",t);
+  ierr = VecDestroy(r_vec); CHKERRQ(ierr);
+  ierr = VecDestroy(z_vec); CHKERRQ(ierr);
+
+  ierr = VecDestroy(p_vec); CHKERRQ(ierr);
+  ierr = VecDestroy(q_vec); CHKERRQ(ierr);
 
   return 0;
 }
