@@ -23,7 +23,7 @@ int prec_setup(MPI_Comm comm,Mat A,PC the_pc)
       ierr = PCSetType(very_local_pc,PCLU); CHKERRQ(ierr);
     }
     ierr = PCParallelSubdomainPipelineSetType
-              (intface_pc,PIPELINE_SEQUENTIAL);
+              (intface_pc,PIPELINE_SEQUENTIAL,(PetscObject)PETSC_NULL);
     CHKERRQ(ierr);
 /*
     ierr = PCSetType(intface_pc,PCAdditiveSchwarz); CHKERRQ(ierr);
