@@ -9,7 +9,7 @@ int prec_setup(MPI_Comm comm,Mat A,PC the_pc)
 
   MPI_Comm_rank(comm,&mytid);
 
-  if (!mytid) printf("Setting up Generalised Block SSOR pc\n");
+  PetscPrintf(comm,"Setting up Generalised Block SSOR pc\n");
   /* ---------------------------------- *
    * ---- here is the crucial part ---- *
    * ---------------------------------- */
@@ -23,7 +23,8 @@ int prec_setup(MPI_Comm comm,Mat A,PC the_pc)
   }
 
   ierr = PCGenBlockSSORSetGlobalFactorisation(the_pc); CHKERRQ(ierr);
-  ierr = PCParallelSubdomainPipelineSetType(the_pc,PIPELINE_SEQUENTIAL);
+  ierr = PCParallelSubdomainPipelineSetType
+    (the_pc,PIPELINE_SEQUENTIAL,(PetscObject)PETSC_NULL);
   CHKERRQ(ierr);
 
   /* ---------------------------------- */
