@@ -13,9 +13,8 @@ int prec_setup(MPI_Comm comm,Mat A,PC the_pc)
   /* ---------------------------------- *
    * ---- here is the crucial part ---- *
    * ---------------------------------- */
-  if (!mytid) printf("Setting up JACOBI pc %d\n",pc_name);
+  PetscPrintf(comm,"Setting up JACOBI pc\n");
   ierr = PCSetType(the_pc,pc_name); CHKERRQ(ierr);
-  /* ---------------------------------- */
 
   ierr = ParPreSetup(comm,A,the_pc); CHKERRQ(ierr);
 
