@@ -10,8 +10,8 @@ int prec_setup(MPI_Comm comm,Mat A,PC the_pc)
 
   MPI_Comm_rank(comm,&mytid);
 
-  if (!mytid) printf("Setting up Multiplicative Schwarz pc %d\n",
-		     (int)pc_name);
+  PetscPrintf(comm,"Setting up Multiplicative Schwarz pc \n");
+
   /* ---------------------------------- *
    * ---- here is the crucial part ---- *
    * ---------------------------------- */
@@ -21,7 +21,8 @@ int prec_setup(MPI_Comm comm,Mat A,PC the_pc)
   ierr = PCSchwarzSetHaloSize(the_pc,1); CHKERRQ(ierr);
 
   /* Declare global communication scheme */
-  ierr = PCParallelSubdomainPipelineSetType(the_pc,PIPELINE_SEQUENTIAL);
+  ierr = PCParallelSubdomainPipelineSetType
+    (the_pc,PIPELINE_SEQUENTIAL,(PetscObject)PETSC_NULL);
 
   /* Declare local solve */
   {
