@@ -1,27 +1,21 @@
 #include <stdio.h>
-#include "petsc.h"
-#include "mat.h"
-#include "pc.h"
+#include "./examples.h"
 
-int prec_setup(MPI_Comm comm,Mat A,PC *B)
+#undef __FUNC__
+#define __FUNC__ "prec_setup"
+int prec_setup(MPI_Comm comm,Mat A,PC the_pc)
 {
-  PC the_pc;
   PCType pc_name = PCNONE;
   int mytid,ierr;
 
   MPI_Comm_rank(comm,&mytid);
-  ierr = PCCreate(comm,&the_pc); CHKERRQ(ierr);
   /* ---------------------------------- *
    * ---- here is the crucial part ---- *
    * ---------------------------------- */
-  if (!mytid) printf("Setting up NONE pc %d\n",pc_name);
+  PetscPrintf(comm,"Setting up NONE pc \n");
   ierr = PCSetType(the_pc,pc_name); CHKERRQ(ierr);
-  /* ---------------------------------- */
 
   ierr = ParPreSetup(comm,A,the_pc); CHKERRQ(ierr);
-  ierr = PCSetUp(the_pc); CHKERRQ(ierr);
-
-  *B = the_pc;
 
   return 0;
 }
