@@ -6,10 +6,12 @@
 #include "mat.h"
 #include "parpre_pipeline.h"
 
+typedef enum {MatNeedsCreating=1,MatAlreadyCreated=2} MatReuse;
+
 typedef struct _MatGatherCtx* MatGatherCtx;
 
 extern int  MatCreateScatterPipeline
-(Mat base_mat,PipelineType pipe_type,
+(Mat base_mat,PipelineType pipe_type,PetscObject pipe_obj,
  Vec *border_vec,VecPipeline *main_pipe);
 
 extern int MatGatherCtxCreate(Mat mat,IS wanted,MatGatherCtx *rgs);
