@@ -6,7 +6,8 @@
 typedef enum {PIPELINE_DOWN=0,PIPELINE_UP=1
 	    } PipelineDirection;
 typedef enum {PIPELINE_NONE=1, PIPELINE_SEQUENTIAL=2,
-		PIPELINE_REDBLACK=3, PIPELINE_MULTICOLOUR=4
+	      PIPELINE_REDBLACK=3, PIPELINE_MULTICOLOUR=4,
+	      PIPELINE_USER=5
 	    } PipelineType;
 
 typedef struct _VecPipeline*  VecPipeline;
@@ -16,16 +17,15 @@ typedef struct _PipelineObject* PipelineObject;
 extern int VecPipelineCreate(MPI_Comm comm,
 			     Vec xin,IS ix,Vec yin,IS iy,VecPipeline *newctx);
 extern int VecPipelineSetup(VecPipeline ctx);
-extern int PCCustomPipelineSetFromOptions(PC pc);
 extern int VecPipelineDestroy(VecPipeline ctx);
 extern int VecPipelineSetType(VecPipeline ctx,PipelineType typ,PetscObject x);
 extern int VecPipelineBegin(Vec,Vec,InsertMode,
 			    ScatterMode,PipelineDirection,VecPipeline);
 extern int VecPipelineEnd(Vec,Vec,InsertMode,
 			  ScatterMode,PipelineDirection,VecPipeline); 
+extern int VecPipelineIsStartOfPipe(VecPipeline ctx,
+				    PipelineDirection pmode,int *flg);
 
 extern int VecPipelineView(VecPipeline pipe,Viewer viewer);
-extern int VecScatterCopyToPipeline(VecScatter in,VecPipeline *out);
-extern int IsProcYes(int fun);
 
 #endif
