@@ -5,6 +5,6 @@
 
 typedef enum 
 {PETSC_NOT_SET_VALUES, PETSC_INSERT_VALUES, 
- PETSC_ADD_VALUES, MAX_VALUES} ParPreInsertMode;
+ PETSC_ADD_VALUES, PETSC_MAX_VALUES} ParPreInsertMode;
 
 #endif
