@@ -4,49 +4,74 @@
 #include "src/vec/utils/pipeline_impl.h"
 #include "parpre_mat.h"
 #include "parpre_pc.h"
+#include "src/mat/matimpl.h"
 #include "src/mat/parpre_matimpl.h"
-#include "src/pc/pcextra.h"
 
-/* VE functions added from here on down */
+extern int IntSort(int *ar,int len);
+extern int IsProcYes(int fun);
+
+#undef __FUNC__
+#define __FUNC__ "MatGetRowLen_MPIAIJ"
 int MatGetRowLen_MPIAIJ(Mat mat,int row,int *len)
 {
   Mat_MPIAIJ *Aij = (Mat_MPIAIJ *) mat->data;
   Mat dia = Aij->A, off = Aij->B;
   Mat_SeqAIJ *aij = (Mat_SeqAIJ *) dia->data, *bij = (Mat_SeqAIJ *) off->data;
 
+  PetscFunctionBegin;
   if ( (row<Aij->rstart) | (row>=Aij->rend) )
     SETERRQ(1,0,"GetRowLen only for local rows");
   row -= Aij->rstart;
   *len = aij->i[row+1] - aij->i[row] + bij->i[row+1] - bij->i[row];
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatMaxRowLen_MPIAIJ"
 int MatMaxRowLen_MPIAIJ(Mat A, int *rowlen)
 {
   Mat_MPIAIJ *Aij = (Mat_MPIAIJ *) A->data;
   Mat dia = Aij->A, off = Aij->B;
-  Mat_SeqAIJ *aij = (Mat_SeqAIJ *) dia->data, *bij = (Mat_SeqAIJ *) off->data;
-
+  Mat_SeqAIJ *aij = (Mat_SeqAIJ *) dia->data;
   int rl=0,rlt,i;
 
-  for ( i=0; i<aij->m; i++ ) {
-    rlt = aij->i[i+1] - aij->i[i] + bij->i[i+1] - bij->i[i];
-    if (rlt>rl) rl=rlt;
+  PetscFunctionBegin;
+  if (off) {
+    Mat_SeqAIJ *bij = (Mat_SeqAIJ *) off->data;
+
+    for ( i=0; i<aij->m; i++ ) {
+      rlt = aij->i[i+1] - aij->i[i] + bij->i[i+1] - bij->i[i];
+      if (rlt>rl) rl=rlt;
+    }
+  } else {
+    for ( i=0; i<aij->m; i++ ) {
+      rlt = aij->i[i+1] - aij->i[i];
+      if (rlt>rl) rl=rlt;
+    }
   }
   *rowlen = rl;
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatMaxRowOffDiagElement_MPIAIJ"
 int MatMaxRowOffDiagElement_MPIAIJ(Mat A,Vec e)
 {
   int rstart,rend,Row,ierr;
+
+  PetscFunctionBegin;
   ierr = MatGetOwnershipRange(A,&rstart,&rend); CHKERRQ(ierr);
   for (Row=rstart; Row<rend; Row++) {
     int ncols,*cols,iCol; Scalar *vals,m=0.0;
     ierr = MatGetRow(A,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
     for (iCol=0; iCol<ncols; iCol++) {
       if (cols[iCol]!=Row)
+#if defined(USE_PETSC_COMPLEX)
+	if (PetscAbsScalar(vals[iCol])>PetscAbsScalar(m))
+	  m = PetscAbsScalar(vals[iCol]);
+#else
 	if (fabs(vals[iCol])>m) m = fabs(vals[iCol]);
+#endif
     }
     ierr = VecSetValues(e,1,&Row,&m,INSERT_VALUES); CHKERRQ(ierr);
     ierr = MatRestoreRow(A,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
@@ -54,19 +79,23 @@ int MatMaxRowOffDiagElement_MPIAIJ(Mat A,Vec e)
   ierr = VecAssemblyBegin(e); CHKERRQ(ierr);
   ierr = VecAssemblyEnd(e); CHKERRQ(ierr);
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatMaxColOffDiagElement_MPIAIJ"
 int MatMaxColOffDiagElement_MPIAIJ(Mat A,Vec e)
 {
   int rstart,rend,Row,ierr;
+
+  PetscFunctionBegin;
   ierr = MatGetOwnershipRange(A,&rstart,&rend); CHKERRQ(ierr);
   for (Row=rstart; Row<rend; Row++) {
     int ncols,*cols,iCol; Scalar *vals;
     ierr = MatGetRow(A,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
     for (iCol=0; iCol<ncols; iCol++) {
       if (cols[iCol]!=Row) {
-	Scalar v = fabs(vals[iCol]);
+	Scalar v = PetscAbsScalar(vals[iCol]);
 	ierr = VecSetValues(e,1,cols+iCol,&v,/*MAX*/ADD_VALUES);
 	CHKERRQ(ierr);
       }
@@ -76,9 +105,154 @@ int MatMaxColOffDiagElement_MPIAIJ(Mat A,Vec e)
   ierr = VecAssemblyBegin(e); CHKERRQ(ierr);
   ierr = VecAssemblyEnd(e); CHKERRQ(ierr);
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatMatAdd_MPIAIJ"
+int MatMatAdd_MPIAIJ(Mat A,Mat B,MatReuse use,Mat *C)
+{
+  Mat res;
+  MPI_Comm comm = A->comm;
+  int iRow,A_local,A_start,A_end,A_global,idum,ierr;
+
+  PetscFunctionBegin;
+
+  {/* consistency check */
+    int A_i,B_i,B_j;
+    ierr = MatGetSize(A,&A_i,&A_global); CHKERRQ(ierr);
+    ierr = MatGetSize(B,&B_i,&B_j); CHKERRQ(ierr);
+    if (!( (A_i==B_i) && (A_global==B_j) )) {
+      printf("A=%dx%d, B=%d,%d\n",A_i,A_global,B_i,B_j);
+      SETERRQ(1,0,"MatMatAdd_MPIAIJ: A,B global size mismatch");
+    }
+    if (use==MatAlreadyCreated) {
+      ierr = MatGetSize(*C,&B_i,&B_j); CHKERRQ(ierr);
+      if (!( (A_i==B_i) && (A_global==B_j) )) {
+	SETERRQ(1,0,"MatMatAdd_MPIAIJ: A,C global size mismatch");
+      }
+    }
+  }
+
+  ierr = MatGetOwnershipRange(A,&A_start,&A_end); CHKERRQ(ierr);
+  A_local = A_end-A_start;
+
+  if (use==MatNeedsCreating) {
+    int *band;
+    /* (over)estimate resulting bandwidth */
+    band = (int *) PetscMalloc((A_local+1)*sizeof(int)); CHKPTRQ(band);
+    for (iRow=0; iRow<A_local; iRow++) {
+      int Row=A_start+iRow,ncols;
+      ierr = MatGetRow(A,Row,&ncols,PETSC_NULL,PETSC_NULL);
+      CHKERRQ(ierr);
+    band[iRow] = ncols;
+    ierr = MatRestoreRow(A,Row,&ncols,PETSC_NULL,PETSC_NULL);
+    CHKERRQ(ierr);
+    
+    ierr = MatGetRow(B,Row,&ncols,PETSC_NULL,PETSC_NULL);
+    CHKERRQ(ierr);
+    band[iRow] += ncols;
+    ierr = MatRestoreRow(B,Row,&ncols,PETSC_NULL,PETSC_NULL);
+    CHKERRQ(ierr);
+    }
+    
+    /* create matrix and fill in */
+    ierr = MatCreateMPIAIJ
+      (comm,A_local,PETSC_DECIDE,PETSC_DECIDE,A_global,
+       PETSC_NULL,band,PETSC_NULL,band,&res);
+    CHKERRQ(ierr);
+    PetscFree(band);
+  } else res = *C;
+
+  for (iRow=0; iRow<A_local; iRow++) {
+    int Row=A_start+iRow,ncols,*cols; Scalar *vals;
+    
+    ierr = MatGetRow(A,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+    ierr = MatSetValues(res,1,&Row,ncols,cols,vals,ADD_VALUES);
+    CHKERRQ(ierr);
+    ierr = MatRestoreRow(A,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+    
+    ierr = MatGetRow(B,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+    ierr = MatSetValues(res,1,&Row,ncols,cols,vals,ADD_VALUES);
+    CHKERRQ(ierr);
+    ierr = MatRestoreRow(B,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+  }
+  ierr = MatAssemblyBegin(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
+  ierr = MatAssemblyEnd(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
+  *C = res;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNC__
+#define __FUNC__ "MatMatSub_MPIAIJ"
+int MatMatSub_MPIAIJ(Mat A,Mat B,Mat *C)
+{
+  Mat res;
+  MPI_Comm comm = A->comm;
+  int *band,iRow,A_local,A_start,A_end,A_global,idum,ierr;
+
+  PetscFunctionBegin;
+
+  {/* consistency check */
+    int A_i,B_i,B_j;
+    ierr = MatGetSize(A,&A_i,&A_global); CHKERRQ(ierr);
+    ierr = MatGetSize(B,&B_i,&B_j); CHKERRQ(ierr);
+    if (!( (A_i==B_i) && (A_global==B_j) )) {
+      printf("A=%dx%d, B=%d,%d\n",A_i,A_global,B_i,B_j);
+      SETERRQ(1,0,"MatMatSub _MPIAIJ: Global size mismatch");
+    }
+  }
+
+  ierr = MatGetOwnershipRange(A,&A_start,&A_end); CHKERRQ(ierr);
+  A_local = A_end-A_start;
+
+  /* (over)estimate resulting bandwidth */
+  band = (int *) PetscMalloc((A_local+1)*sizeof(int)); CHKPTRQ(band);
+  for (iRow=0; iRow<A_local; iRow++) {
+    int Row=A_start+iRow,ncols;
+    ierr = MatGetRow(A,Row,&ncols,PETSC_NULL,PETSC_NULL);
+    CHKERRQ(ierr);
+    band[iRow] = ncols;
+    ierr = MatRestoreRow(A,Row,&ncols,PETSC_NULL,PETSC_NULL);
+    CHKERRQ(ierr);
+    
+    ierr = MatGetRow(B,Row,&ncols,PETSC_NULL,PETSC_NULL);
+    CHKERRQ(ierr);
+    band[iRow] += ncols;
+    ierr = MatRestoreRow(B,Row,&ncols,PETSC_NULL,PETSC_NULL);
+    CHKERRQ(ierr);
+  }
+
+  /* create matrix and fill in */
+  ierr = MatCreateMPIAIJ
+    (comm,A_local,PETSC_DECIDE,PETSC_DECIDE,A_global,
+     PETSC_NULL,band,PETSC_NULL,band,&res);
+  CHKERRQ(ierr);
+  PetscFree(band);
+
+  for (iRow=0; iRow<A_local; iRow++) {
+    int Row=A_start+iRow,ncols,*cols,icol; Scalar *vals;
+    
+    ierr = MatGetRow(A,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+    ierr = MatSetValues(res,1,&Row,ncols,cols,vals,ADD_VALUES);
+    CHKERRQ(ierr);
+    ierr = MatRestoreRow(A,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+    
+    ierr = MatGetRow(B,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+    for (icol=0; icol<ncols; icol++) vals[icol]=-vals[icol];
+    ierr = MatSetValues(res,1,&Row,ncols,cols,vals,ADD_VALUES);
+    CHKERRQ(ierr);
+    for (icol=0; icol<ncols; icol++) vals[icol]=-vals[icol];
+    ierr = MatRestoreRow(B,Row,&ncols,&cols,&vals); CHKERRQ(ierr);
+  }
+  ierr = MatAssemblyBegin(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
+  ierr = MatAssemblyEnd(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
+  *C = res;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNC__
+#define __FUNC__ "MatMatMult_MPIAIJ"
 int MatMatMult_MPIAIJ(Mat A,Mat B,Mat *C)
 {
   Mat_MPIAIJ  *Aij = (Mat_MPIAIJ *) A->data;
@@ -88,29 +262,31 @@ int MatMatMult_MPIAIJ(Mat A,Mat B,Mat *C)
   int A_start,A_local,A_global,B_global;
   int A_off = ((Mat_SeqAIJ*) Aij->B->data)->n;
 
-  {/* consistency check */
-    int A_dum,B_dum;
+  PetscFunctionBegin;
+
+  /* get various sizes, and perform consistency check */
+  {
+    int A_dum,B_dum,A_end;
     ierr = MatGetSize(A,&A_global,&A_dum); CHKERRQ(ierr);
     ierr = MatGetSize(B,&B_dum,&B_global); CHKERRQ(ierr);
-    if (!(A_dum==B_dum)) SETERRQ(1,0,"MatMatMult: Global size mismatch");
+    if (!(A_dum==B_dum))
+      SETERRQ(1,0,"MatMatMult: Global inner dimension mismatch");
+    ierr = MatGetOwnershipRange(A,&A_start,&A_end); CHKERRQ(ierr);
+    A_local = A_end-A_start;
   }
 
-  /* get various sizes, and allocate the result matrix */
-  {
-    int A_jlocal,B_local,B_jlocal,idum;
-    ierr = MatGetOwnershipRange(A,&A_start,&idum); CHKERRQ(ierr);
-    ierr = MatGetLocalSize(A,&A_local,&A_jlocal); CHKERRQ(ierr);
-    ierr = MatGetLocalSize(B,&B_local,&B_jlocal); CHKERRQ(ierr);
-    if (A_global!=B_global) {
-      /* is this case ever appropriate? */
-printf(">>>> Strange case: MatMatMult_MPIAIJ <<<<\n");
-      ierr = MatCreateMPIAIJ(comm,PETSC_DECIDE,PETSC_DECIDE,A_global,B_global,
-			     0,0,0,0,&res); CHKERRQ(ierr);
-    }
+  if (A_global==B_global) {
+    ierr = MatCreateMPIAIJ
+      (comm,A_local,A_local,PETSC_DECIDE,PETSC_DECIDE,0,0,0,0,&res);
+    CHKERRQ(ierr);
+  } else {
+    ierr = MatCreateMPIAIJ
+      (comm,A_local,PETSC_DECIDE,PETSC_DECIDE,B_global,0,0,0,0,&res);
+    CHKERRQ(ierr);
   }
 
   {
-    Mat *sub,tmp1,tmp2;
+    Mat *sub,tmp1,tmp2,tmp3;
     IS A_home,A_away,B_total,IS_A[2],IS_B[2];
     int *band, iRow;
 
@@ -135,50 +311,25 @@ printf(">>>> Strange case: MatMatMult_MPIAIJ <<<<\n");
       ierr = MatDestroy(off_factor); CHKERRQ(ierr);
     }
     PetscFree(sub);
-    /* (over)estimate resulting bandwidth */
-    band = (int *) PetscMalloc((A_local+1)*sizeof(int)); CHKPTRQ(band);
-    for (iRow=0; iRow<A_local; iRow++) {
-      int ncols;
-      ierr = MatGetRow(tmp1,iRow,&ncols,PETSC_NULL,PETSC_NULL);
-      CHKERRQ(ierr);
-      band[iRow] = ncols;
-      ierr = MatRestoreRow(tmp1,iRow,&ncols,PETSC_NULL,PETSC_NULL);
-      CHKERRQ(ierr);
-      
-      ierr = MatGetRow(tmp2,iRow,&ncols,PETSC_NULL,PETSC_NULL);
-      CHKERRQ(ierr);
-      band[iRow] += ncols;
-      ierr = MatRestoreRow(tmp2,iRow,&ncols,PETSC_NULL,PETSC_NULL);
-      CHKERRQ(ierr);
-    }
-    /* create matrix and fill in */
-    ierr = MatCreateMPIAIJ
-      (comm,A_local,A_local,PETSC_DECIDE,PETSC_DECIDE,
-       PETSC_NULL,band,PETSC_NULL,band,&res);
-    CHKERRQ(ierr);
-    PetscFree(band);
-    for (iRow=0; iRow<A_local; iRow++) {
-      int Row=A_start+iRow,ncols,*cols; Scalar *vals;
 
+    for (iRow=0; iRow<A_local; iRow++) {
+      int Row=A_start+iRow, ncols,*cols; Scalar *vals;
       ierr = MatGetRow(tmp1,iRow,&ncols,&cols,&vals); CHKERRQ(ierr);
       ierr = MatSetValues(res,1,&Row,ncols,cols,vals,ADD_VALUES);
       CHKERRQ(ierr);
       ierr = MatRestoreRow(tmp1,iRow,&ncols,&cols,&vals); CHKERRQ(ierr);
-
       ierr = MatGetRow(tmp2,iRow,&ncols,&cols,&vals); CHKERRQ(ierr);
       ierr = MatSetValues(res,1,&Row,ncols,cols,vals,ADD_VALUES);
       CHKERRQ(ierr);
       ierr = MatRestoreRow(tmp2,iRow,&ncols,&cols,&vals); CHKERRQ(ierr);
     }
-    ierr = MatDestroy(tmp1); CHKERRQ(ierr);
-    ierr = MatDestroy(tmp2); CHKERRQ(ierr);
+    ierr = MatAssemblyBegin(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
+    ierr = MatAssemblyEnd(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
   }
-  ierr = MatAssemblyBegin(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
-  ierr = MatAssemblyEnd(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
 
   *C = res;
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
 #undef __FUNC__
@@ -188,6 +339,7 @@ int MatrixAij2MpiAbut(int part,Mat in,MPI_Comm comm,Mat *out)
   Mat res;
   int *band,m,n,irow,rstart,rend,ierr;
 
+  PetscFunctionBegin;
   /* get the bandwidth of the matrix to be embedded,
      specifying this as d_nnz/o_nnz is an overestimate */
   ierr = MatGetLocalSize(in,&m,&n); CHKERRQ(ierr);
@@ -215,7 +367,7 @@ int MatrixAij2MpiAbut(int part,Mat in,MPI_Comm comm,Mat *out)
   ierr = MatAssemblyEnd(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
 
   *out = res;
-  return 0;
+  PetscFunctionReturn(0);
 }
 
 #undef __FUNC__
@@ -226,6 +378,7 @@ int MatrixAij2MpiIndex(IS ptr,int m,int n,int M,int N,
   Mat res;
   int irow,ierr, nidx,*idx;
 
+  PetscFunctionBegin;
 printf(">> Using unoptimised routine Aij2MpiIndex <<\n");
   ierr = ISGetSize(ptr,&nidx); CHKERRQ(ierr);
   {
@@ -248,7 +401,7 @@ printf(">> Using unoptimised routine Aij2MpiIndex <<\n");
   ierr = MatAssemblyEnd(res,MAT_FINAL_ASSEMBLY); CHKERRQ(ierr);
 
   *out = res;
-  return 0;
+  PetscFunctionReturn(0);
 }
 
 /****************************************************************
@@ -257,7 +410,7 @@ printf(">> Using unoptimised routine Aij2MpiIndex <<\n");
 #undef __FUNC__
 #define __FUNC__ "MatCreateScatterPipeline"
 int MatCreateScatterPipeline
-(Mat base_mat,PipelineType pipe_type,
+(Mat base_mat,PipelineType pipe_type,PetscObject pipe_obj,
  Vec *border_vec,VecPipeline *main_pipe)
 {
   Mat_MPIAIJ *Aij = (Mat_MPIAIJ *) base_mat->data;
@@ -267,12 +420,13 @@ int MatCreateScatterPipeline
   int local_size,low_border,local_border,local_domain;
   int dum,ierr;
 
+  PetscFunctionBegin;
   ierr = MatGetOwnershipRange(base_mat,&local_domain,&dum); CHKERRQ(ierr);
   local_size = dum-local_domain;
   ierr = VecCreateMPI(comm,local_size,PETSC_DECIDE,&tmp_g); CHKERRQ(ierr);
   ierr = VecAssemblyBegin(tmp_g); CHKERRA(ierr);
   ierr = VecAssemblyEnd(tmp_g); CHKERRA(ierr);
-  local_border= ((Mat_SeqAIJ *)Aij->B->data)->n;
+  local_border= ((Mat_SeqAIJ *)(Aij->B)->data)->n;
   ierr = ISCreateGeneral(MPI_COMM_SELF,local_border,Aij->garray,&is_g);
   CHKERRQ(ierr);
 
@@ -287,7 +441,8 @@ int MatCreateScatterPipeline
   
   ierr = VecPipelineCreate(comm,tmp_g,is_g,bvec,is_l,&pipe);
   CHKERRQ(ierr);
-  ierr = VecPipelineSetType(pipe,pipe_type,(PetscObject)base_mat);
+  /* !!! there was base_mat here. */
+  ierr = VecPipelineSetType(pipe,pipe_type,pipe_obj);
   CHKERRQ(ierr);
   ierr = VecPipelineSetup(pipe); CHKERRQ(ierr);
 
@@ -297,7 +452,7 @@ int MatCreateScatterPipeline
 
   *border_vec = bvec; *main_pipe = pipe;
 
-  return 0;
+  PetscFunctionReturn(0);
 }
   
 /* VE from here on down probably superfluous */
@@ -310,6 +465,8 @@ int MatCreateScatterPipeline
  * into a sequential matrix. There are separate creation
  * and collection routines. The case where the index set "wanted"
  * is null on one or more processors is handled correctly. */
+#undef __FUNC__
+#define __FUNC__ "MatGatherCtxCreate"
 int MatGatherCtxCreate(Mat mat,IS wanted,MatGatherCtx *rgs)
 /* check header, cookie = MATMPIAIJ */
 {
@@ -324,6 +481,7 @@ int MatGatherCtxCreate(Mat mat,IS wanted,MatGatherCtx *rgs)
   int i,ierr,p,send_nrows_tot;
   MatGatherCtx gs;
 
+  PetscFunctionBegin;
   MPI_Comm_size(comm,&numtids);
   MPI_Comm_rank(comm,&mytid);
 
@@ -449,19 +607,24 @@ printf("\n");
   /* free temporaries */
   PetscFree(send_nrows); PetscFree(request_nrows);
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatGatherCtxDestroy"
 int MatGatherCtxDestroy(MatGatherCtx gs)
 {
+  PetscFunctionBegin;
   PetscFree(gs->send_procs_all);
   PetscFree(gs->recv_procs_all);
   PetscFree(gs->send_rows);
   PetscFree(gs->send_rows_ptrs);
   PetscFree(gs->request_row_ptrs);
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatGatherRowsReceive"
 static int MatGatherRowsReceive
 (MatGatherCtx gs,PipelineFunction pf,PetscObject pl,Mat *return_mat)
 {
@@ -470,6 +633,7 @@ static int MatGatherRowsReceive
   int ierr,count,ip;
   int *recv_procs,nrecvs;
 
+  PetscFunctionBegin;
   if (pf) gs->custom_recvs = 1;
   else gs->custom_recvs = 0;
 
@@ -566,9 +730,11 @@ for(ip=0;ip<nrecvs;ip++)printf("%d, ",recv_procs[ip]);printf("\n");
 
   PetscFree(recv_procs);
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatGatherRowsSend"
 static int MatGatherRowsSend
 (MatGatherCtx gs,Mat mat,PipelineFunction pf,PetscObject pl)
 {
@@ -576,6 +742,7 @@ static int MatGatherRowsSend
   int *send_procs,nsends;
   int ip;
 
+  PetscFunctionBegin;
   if (!pf) gs->custom_sends = 0;
   else gs->custom_sends = 1;
   
@@ -673,26 +840,32 @@ for(ip=0;ip<nsends;ip++)printf("%d, ",send_procs[ip]);printf("\n");
   }
 
   PetscFree(send_procs);
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatGatherRows"
 int MatGatherRows(Mat mat,MatGatherCtx gs, Mat *res_mat)
 /* check header, cookie = MATMPIAIJ */
 {
   int ierr;
 
+  PetscFunctionBegin;
   ierr = MatGatherRowsSend(gs,mat,0,0); CHKERRQ(ierr);
   ierr = MatGatherRowsReceive(gs,0,0,res_mat); CHKERRQ(ierr);
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatGatherRowsPipelineBegin"
 int MatGatherRowsPipelineBegin
 (Mat mat,PipelineDirection ptype, MatGatherCtx gs, Mat *ret_mat)
 {
   VecPipeline pipe = gs->vs; PipelineFunction pf;
   int ierr;
 
+  PetscFunctionBegin;
   if (ptype == PIPELINE_UP)
     pf = pipe->dnfn;
   else
@@ -705,9 +878,11 @@ int MatGatherRowsPipelineBegin
     PetscObject pl = pipe->custom_pipe_data;
     ierr = MatGatherRowsReceive(gs,pf,pl,ret_mat);
   }
-  return 0;
+  PetscFunctionReturn(0);
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatGatherRowsPipelineEnd"
 int MatGatherRowsPipelineEnd
 (Mat mat,PipelineDirection ptype, MatGatherCtx gs, Mat *ret_mat)
 {
@@ -715,6 +890,7 @@ int MatGatherRowsPipelineEnd
   PetscObject pl = pipe->custom_pipe_data;
   int ierr,flg;
 
+  PetscFunctionBegin;
   if (ptype == PIPELINE_UP)
     pf = pipe->upfn;
   else
@@ -725,7 +901,7 @@ int MatGatherRowsPipelineEnd
   if (((int)pf) && (!flg))
     {ierr = MatGatherRowsSend(gs,mat,pf,pl); CHKERRQ(ierr);}
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
 /* VE !!! not removed */
@@ -737,6 +913,7 @@ int MatGetSubMatrix_MPIAIJ(Mat A,IS isrow,IS iscol,MatGetSubMatrixCall scall,Mat
   IS all_rows;
   int ierr,nrows;
 
+  PetscFunctionBegin;
   ierr = MatGatherCtxCreate(A,isrow,&get_strip); CHKERRQ(ierr);
   ierr = MatGatherRows(A,get_strip,&wide_mat); CHKERRQ(ierr);
 
@@ -756,6 +933,6 @@ int MatGetSubMatrix_MPIAIJ(Mat A,IS isrow,IS iscol,MatGetSubMatrixCall scall,Mat
     ierr = MatDestroy(wide_mat); CHKERRQ(ierr);
   }
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 */
