@@ -6,6 +6,8 @@
 #include "src/vec/vecimpl.h"
 #include "petsc.h"
 
+#undef __FUNC__
+#define __FUNC__ "MatMultAXBY_AIJ"
 int MatMultAXBY_AIJ(Scalar a, Mat aijin,Vec xx, Scalar b, Vec yy,Vec zz)
 {
   Mat_SeqAIJ *aij = (Mat_SeqAIJ *) aijin->data;
@@ -31,6 +33,8 @@ int MatMultAXBY_AIJ(Scalar a, Mat aijin,Vec xx, Scalar b, Vec yy,Vec zz)
   return 0;
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatMult_AIJ"
 int MatMatMult_AIJ(Mat a, Mat b, Mat *c)
 {
   Mat_SeqAIJ *aij = (Mat_SeqAIJ *) a->data;
@@ -41,7 +45,8 @@ int MatMatMult_AIJ(Mat a, Mat b, Mat *c)
   if ( ! (aij->n==bij->m) )
     SETERRQ(1,0,"MatMatMult_AIJ: Non conforming matrix dimensions");
 
-  nd = aij->nz*bij->nz/(aij->n*bij->m);
+  nd = aij->n*bij->m;
+  if (nd>0) nd = aij->nz*bij->nz/nd; else nd = 0;
   ierr = MatCreateSeqAIJ(a->comm,aij->m,bij->n,nd,PETSC_NULL,c);
   CHKERRQ(ierr);
 
@@ -60,35 +65,47 @@ int MatMatMult_AIJ(Mat a, Mat b, Mat *c)
   return 0;
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatAdd_AIJ"
 int MatMatAdd_AIJ(Mat a, Mat b, Mat c)
 {
   Mat_SeqAIJ *aij = (Mat_SeqAIJ *) a->data;
   Mat_SeqAIJ *bij = (Mat_SeqAIJ *) b->data;
   Mat_SeqAIJ *cij = (Mat_SeqAIJ *) c->data;
+  int ashift = aij->indexshift, bshift = bij->indexshift,
+    cshift = cij->indexshift;
   int row,ierr,ct=0;
 
 #define INFTY aij->n+1
-  if ( ! ( (aij->m==bij->m) & (aij->m==cij->m) & 
-	   (aij->n==bij->n) & (aij->n==cij->n) ) )
+  if ( ! ( (aij->m==bij->m) && (aij->m==cij->m) &&
+	   (aij->n==bij->n) && (aij->n==cij->n) ) )
     SETERRQ(1,0,"MatMatAdd_AIJ: Non conforming matrix dimensions");
 
+printf("Shifts: %d,%d,%d\n",ashift,bshift,cshift);
   for (row=0; row<aij->m; row++) {
-      int i,j,ja,jb,aj,bj; Scalar av,bv,v;
+      int i,j,ja,jb,aj,bj;
+      int a_end = aij->i[row+1]-1+ashift, b_end = bij->i[row+1]-1+ashift;
+      Scalar av,bv,v;
+
       i = row;
-      ja = aij->i[row]-1;
-      jb = bij->i[row]-1;
-
-      while ( (ja<aij->i[row+1]-1) | (jb<bij->i[row+1]-1) ) {
-	if (ja==aij->i[row+1]-1) aj = INFTY;
+      ja = aij->i[row]+ashift;
+      jb = bij->i[row]+bshift;
+printf("first columns: %d,%d\n",ja,jb);
+      while ( (ja<a_end) || (jb<b_end) ) {
+	if (ja==a_end) aj = INFTY;
 	else
-	  {aj = aij->j[ja]-1; av = aij->a[ja];}
-	if (jb==bij->i[row+1]-1) bj = INFTY;
+	  {aj = aij->j[ja]+ashift; av = aij->a[ja];}
+	if (jb==b_end) bj = INFTY;
 	else
-	  {bj = bij->j[jb]-1; bv = bij->a[jb];}
+	  {bj = bij->j[jb]+bshift; bv = bij->a[jb];}
 
-	if (bj<aj) {v = bv; j = bj; jb++;}
-	if (bj==aj) {v = av+bv; j = bj; ja++; jb++;}
-	if (bj>aj) {v = av; j = aj; ja++;}
+	if (bj<aj) {
+	  v = bv; j = bj; jb++;
+	} else if (bj==aj) {
+	  v = av+bv; j = bj; ja++; jb++;
+	} else if (bj>aj) {
+	  v = av; j = aj; ja++;
+	}
 
 	ct++;
 	ierr = MatSetValues(c,1,&i,1,&j,&v,INSERT_VALUES); CHKERRQ(ierr);
@@ -100,6 +117,8 @@ int MatMatAdd_AIJ(Mat a, Mat b, Mat c)
   return 0;
 }
 
+#undef __FUNC__
+#define __FUNC__ "MatSubtract_AIJ"
 int MatMatSubtract_AIJ(Mat a, Mat b, Mat *c)
 {
   Mat_SeqAIJ *aij = (Mat_SeqAIJ *) a->data;
@@ -143,6 +162,8 @@ int MatMatSubtract_AIJ(Mat a, Mat b, Mat *c)
   return 0;
 }
 
+#undef __FUNC__
+#define __FUNC__ "Transpose_AIJ"
 int MatTranspose_AIJ(Mat a, Mat *b)
 {
   Mat_SeqAIJ *aij = (Mat_SeqAIJ *) a->data;
@@ -164,6 +185,8 @@ int MatTranspose_AIJ(Mat a, Mat *b)
 }
 
 #include "sles.h"
+#undef __FUNC__
+#define __FUNC__ "SolveMat_AIJ"
 int MatSolveMat_AIJ(SLES solver,Mat off_mat,Mat *off_mult)
 {
   Mat off_trans;
