@@ -29,5 +29,7 @@ extern int MatMaxRowLen_MPIAIJ(Mat A, int *rowlen);
 extern int MatMaxRowOffDiagElement_MPIAIJ(Mat A,Vec e);
 extern int MatMaxColOffDiagElement_MPIAIJ(Mat A,Vec e);
 extern int MatMatMult_MPIAIJ(Mat A,Mat B,Mat *C);
+extern int MatMatAdd_MPIAIJ(Mat A,Mat B,MatReuse use,Mat *C);
+extern int MatMatSub_MPIAIJ(Mat A,Mat B,Mat *C);
 
 #endif
