@@ -11,7 +11,6 @@
 #include "parpre_mat.h"
 #include "parpre_mat.h"
 #include "src/mat/parpre_matimpl.h"
-#include "src/pc/pcextra.h"
 #include "options.h"
 #include "src/mat/impls/aij/mpi/mpiaij.h"
 
@@ -317,7 +316,8 @@ static int DDBlocks
   {
     SLES local_sles;
     ierr = PCParallelGetLocalSLES(pc,&local_sles); CHKERRQ(ierr);
-    ierr = SLESSetOperators(local_sles,pc_data->C11,pc_data->C11,0);
+    ierr = SLESSetOperators(local_sles,pc_data->C11,pc_data->C11,
+			    (MatStructure)0);
     CHKERRQ(ierr);
   }
 
@@ -331,7 +331,7 @@ static int DDBlocks
 #define __FUNC__ "PCSetup_DomainDecomp"
 static int PCSetup_DomainDecomp(PC pc)
 {
-  Mat base_mat = pc->mat, from_interface,to_interface,interface_mat;
+  Mat base_mat = pc->pmat, from_interface,to_interface,interface_mat;
   MPI_Comm    comm = base_mat->comm;
   PC_DDecomp_struct *pc_data = (PC_DDecomp_struct *) pc->data;
   IS g_edge_set,g_intl_set;
@@ -439,7 +439,7 @@ static int PCSetup_DomainDecomp(PC pc)
 
     ierr = SLESSetOperators
       (pc_data->interface_method,pc_data->interface_system,
-       pc_data->interface_system,0);
+       pc_data->interface_system,(MatStructure)0);
     CHKERRQ(ierr);
     ierr = SLESGetPC(pc_data->interface_method,&local_pc); CHKERRQ(ierr);
     ierr = PCSetVector(local_pc,pc_data->interface_vector); CHKERRQ(ierr);
@@ -529,9 +529,8 @@ static int PCApply_DomainDecomp(PC pc,Vec x,Vec y)
 
 #undef __FUNC__
 #define __FUNC__ "PCDomainDecomp_View"
-static int PCDomainDecomp_View(PetscObject obj,Viewer viewer)
+static int PCDomainDecomp_View(PC pc,Viewer viewer)
 {
-  PC pc = (PC) obj;
   PC_DDecomp_struct *pc_data = (PC_DDecomp_struct *) pc->data;
   int ierr;
 
@@ -565,9 +564,8 @@ static int PCSetFromOptions_DomainDecomp(PC pc)
 
 #undef __FUNC__
 #define __FUNC__ "PCDestroy_DDecomp"
-static int PCDestroy_DDecomp(PetscObject obj)
+static int PCDestroy_DDecomp(PC pc)
 {
-  PC pc = (PC) obj;
   SLES local_sles;
   PC_DDecomp_struct *pc_data = (PC_DDecomp_struct *) pc->data;
   int ierr;
@@ -614,10 +612,10 @@ int PCCreate_DomainDecomp(PC pc)
   pc->apply     = PCApply_DomainDecomp;
   pc->applyrich = 0;
   pc->destroy   = PCDestroy_DDecomp;
-  pc->setfrom   = PCSetFromOptions_DomainDecomp;
+  pc->setfromoptions   = PCSetFromOptions_DomainDecomp;
   pc->printhelp = 0;
   pc->setup     = PCSetup_DomainDecomp;
-  pc->type      = PCDomainDecomp;
+  /*  pc->type      = PCDomainDecomp;*/
   pc->view      = PCDomainDecomp_View;
 
   /* create subsolvers for the interface system and interior */
