@@ -16,6 +16,7 @@
 
 #define CHUNCKSIZE   100
 
+extern int PCCustomPipelineSetFromOptions(PC pc);
 extern int MatTranspose_AIJ(Mat a, Mat *b);
 extern int ParPreTraceBackErrorHandler
     (int,char*,char*,char*,int,int,char*,void*);
@@ -24,6 +25,7 @@ typedef struct {
   PCParallelSubdomainStruct subdomains;
   int global_factorisation;
   Vec local1,local2,border_vec,global_vec,global_vec2;
+  FillMethod fillmethod;
 } PC_GenBlockSSOR_struct;
 
 /****************************************************************
@@ -35,8 +37,9 @@ typedef struct {
 int PCGenBlockSSORSetNoGlobalFactorisation(PC pc)
 {
   PC_GenBlockSSOR_struct *pc_data = (PC_GenBlockSSOR_struct *) pc->data;
+  PetscFunctionBegin;
   pc_data->global_factorisation = 0;
-  return 0;
+  PetscFunctionReturn(0);
 }
 
 #undef __FUNC__
@@ -44,9 +47,19 @@ int PCGenBlockSSORSetNoGlobalFactorisation(PC pc)
 int PCGenBlockSSORSetGlobalFactorisation(PC pc)
 {
   PC_GenBlockSSOR_struct *pc_data = (PC_GenBlockSSOR_struct *) pc->data;
-  /*printf(">>>> Warning: global GBSSOR factorisation does not work <<<<\n");*/
+  PetscFunctionBegin;
   pc_data->global_factorisation = 1;
-  return 0;
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNC__
+#define __FUNC__ "PCGenBlockSSORSetFillMethod"
+int PCGenBlockSSORSetFillMethod(PC pc,FillMethod fillmethod)
+{
+  PC_GenBlockSSOR_struct *pc_data = (PC_GenBlockSSOR_struct *) pc->data;
+  PetscFunctionBegin;
+  pc_data->fillmethod = fillmethod;
+  PetscFunctionReturn(0);
 }
 
 /****************************************************************
@@ -55,14 +68,14 @@ int PCGenBlockSSORSetGlobalFactorisation(PC pc)
 
 #undef __FUNC__
 #define __FUNC__ "BlockFillDiag"
-static int BlockFillDiag(Mat dia_mat,Mat fill_band,
+static int BlockFillDiag(Mat dia_mat,Mat fill_band,FillMethod fill,
 			 int r0,int rs,Mat *fill_dia)
 { 
   Mat fill_block;
   Mat *res;
   IS rows_abs,rows_rel;
   int ierr;
-  
+
   /* extract relevant part */
   ierr = ISCreateStride
     (MPI_COMM_SELF,rs,0,1,&rows_rel); CHKERRQ(ierr);
@@ -105,36 +118,50 @@ printf("subdomains system:\n"); MatView(fill_dia,0);
 #define __FUNC__ "PCSetup_GBSSOR"
 int PCSetup_GBSSOR(PC pc)
 {
-  Mat base_mat = pc->mat;
+  Mat base_mat = pc->pmat;
   PC_GenBlockSSOR_struct *pc_data = (PC_GenBlockSSOR_struct *) pc->data;
-  int ierr;
+  int ierr,np;
 
   ierr = PetscPushErrorHandler
-    (&ParPreTraceBackErrorHandler,MPI_COMM_WORLD);
+    (&ParPreTraceBackErrorHandler,0);
   CHKERRQ(ierr);
 
+  MPI_Comm_size(pc->comm,&np);
+  if (np==1) {
+    ierr = BlockSetPrec(pc,base_mat); CHKERRQ(ierr);
+    return 0;
+  }
   {
     VecPipeline main_pipe;
     PipelineType pipe_type;
+    PetscObject pipe_obj;
 
     ierr = PCParallelSubdomainPipelineGetType(pc,&pipe_type); CHKERRQ(ierr);
+    ierr = PCParallelSubdomainPipelineGetObject(pc,&pipe_obj); CHKERRQ(ierr);
     if (pipe_type<PIPELINE_NONE) SETERRQ(1,(int)pipe_type,"Invalid pipe type");
     ierr = MatCreateScatterPipeline
-      (base_mat,pipe_type,&(pc_data->border_vec),&main_pipe);
+      (base_mat,pipe_type,pipe_obj,&(pc_data->border_vec),&main_pipe);
     CHKERRQ(ierr);
     ierr = PCParallelSubdomainsSetPipeline(pc,main_pipe); CHKERRQ(ierr);
   }
 
-  { /* create local vectors */
-    int len,dum;
+  /* create local vectors */
+  {
+    int len,dum; Scalar *a;
     ierr = MatGetLocalSize(base_mat,&len,&dum); CHKERRQ(ierr);
     ierr = VecCreateSeq(MPI_COMM_SELF,len,&(pc_data->local1)); CHKERRQ(ierr);
+    ierr = VecGetArray(pc_data->local1,&a); CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray
+      (base_mat->comm,len,PETSC_DECIDE,a,&(pc_data->global_vec));
+    CHKERRQ(ierr);
+    ierr = VecRestoreArray(pc_data->local1,&a); CHKERRQ(ierr);
+
     ierr = VecDuplicate(pc_data->local1,&(pc_data->local2)); CHKERRQ(ierr);
-    ierr = VecCreateMPI
-      (base_mat->comm,len,PETSC_DECIDE,&(pc_data->global_vec));
-    CHKERRQ(ierr);
-    ierr = VecDuplicate(pc_data->global_vec,&(pc_data->global_vec2));
+    ierr = VecGetArray(pc_data->local2,&a); CHKERRQ(ierr);
+    ierr = VecCreateMPIWithArray
+      (base_mat->comm,len,PETSC_DECIDE,a,&(pc_data->global_vec2));
     CHKERRQ(ierr);
+    ierr = VecRestoreArray(pc_data->local2,&a); CHKERRQ(ierr);
   }
 
   /* Determine local coefficient matrix */
@@ -173,7 +200,8 @@ printf("fill in\n");MatView(fill_band,0);
 */
       /* Compute a new diagonal block by filling old rows 
        * onto an old diagonal block */
-      ierr = BlockFillDiag(dia_mat,fill_band,Aij->rstart,Aij->m,&fill_dia);
+      ierr = BlockFillDiag(dia_mat,fill_band,pc_data->fillmethod,
+			   Aij->rstart,Aij->m,&fill_dia);
       CHKERRQ(ierr);
 
       ierr = MatDestroy(fill_band); CHKERRQ(ierr);
@@ -216,11 +244,11 @@ static int PCApply_GBSSOR(PC pc,Vec x,Vec y)
   Mat_MPIAIJ *Aij = (Mat_MPIAIJ *) (base_mat->data);
   Mat off_diag = Aij->B;
   int ierr,its;
-  Vec tmp = pc_data->local1, tmq = pc_data->local2;
+  Vec local1 = pc_data->local1, local2 = pc_data->local2;
   SLES local_method;
   PipelineType pipe_type;
 
-  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,MPI_COMM_WORLD);
+  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,0);
   CHKERRQ(ierr);
 
   ierr = PCParallelGetLocalSLES(pc,&local_method); CHKERRQ(ierr);
@@ -231,88 +259,65 @@ static int PCApply_GBSSOR(PC pc,Vec x,Vec y)
 
     Scalar *ps,*qs,*xa,*ya;
     /* redirect input */
-    ierr = VecGetArray(tmp,&ps); CHKERRQ(ierr);
+    ierr = VecGetArray(local1,&ps); CHKERRQ(ierr);
     ierr = VecGetArray(x,&xa); CHKERRQ(ierr);
-    ierr = VecPlaceArray(tmp,xa); CHKERRQ(ierr);
+    ierr = VecPlaceArray(local1,xa); CHKERRQ(ierr);
     /* redirect output */
-    ierr = VecGetArray(tmq,&qs); CHKERRQ(ierr);
+    ierr = VecGetArray(local2,&qs); CHKERRQ(ierr);
     ierr = VecGetArray(y,&ya); CHKERRQ(ierr);
-    ierr = VecPlaceArray(tmq,ya); CHKERRQ(ierr);
+    ierr = VecPlaceArray(local2,ya); CHKERRQ(ierr);
     /* solve */
-    ierr = SLESSolve(local_method,tmp,tmq,&its); CHKERRQ(ierr);
+    ierr = SLESSolve(local_method,local1,local2,&its); CHKERRQ(ierr);
     /* undirect */
-    ierr = VecPlaceArray(tmp,ps); CHKERRQ(ierr);
-    ierr = VecPlaceArray(tmq,qs); CHKERRQ(ierr);    
+    ierr = VecPlaceArray(local1,ps); CHKERRQ(ierr);
+    ierr = VecPlaceArray(local2,qs); CHKERRQ(ierr);
   } else {
 
     /* >>>> solve in a pipeline <<<< */
 
-    Vec global = pc_data->global_vec, global2 = pc_data->global_vec2;
+    Vec global1 = pc_data->global_vec, global2 = pc_data->global_vec2;
     Vec border = pc_data->border_vec;
     VecPipeline main_pipe;
     Scalar zero = 0.0, mone = -1.0, one = 1.0;
+    int flg;
 
     ierr = PCParallelSubdomainsGetPipeline(pc,&main_pipe); CHKERRQ(ierr);
 
-    /* forward solve from x into global */
+    /* forward solve from global1 into global2,
+       where first global1=x-off*border */
     ierr = VecSet(&zero,border); CHKERRQ(ierr);
     ierr = VecPipelineBegin
       (global2,border, INSERT_VALUES, SCATTER_FORWARD,PIPELINE_UP,main_pipe);
     CHKERRQ(ierr);
     
-    ierr = MatMultAXBY_AIJ(mone,off_diag,border,one,x,global); CHKERRQ(ierr);
-    {
-      Scalar *x_ar,*y_ar,*p_ar,*q_ar;
+    ierr = MatMultAXBY_AIJ(mone,off_diag,border,one,x,global1); CHKERRQ(ierr);
 
-      /* redirect input */
-      ierr = VecGetArray(tmp,&p_ar); CHKERRQ(ierr);
-      ierr = VecGetArray(global,&x_ar); CHKERRQ(ierr);
-      ierr = VecPlaceArray(tmp,x_ar); CHKERRQ(ierr);
-      /* redirect output */
-      ierr = VecGetArray(tmq,&q_ar); CHKERRQ(ierr);
-      ierr = VecGetArray(global2,&y_ar); CHKERRQ(ierr);
-      ierr = VecPlaceArray(tmq,y_ar); CHKERRQ(ierr);
+    /* solve */
+    ierr = SLESSolve(local_method,local1,local2,&its); CHKERRQ(ierr);
+    
+    ierr = VecPipelineEnd
+      (global2,border, INSERT_VALUES, SCATTER_FORWARD,PIPELINE_UP,main_pipe);
+    CHKERRQ(ierr);
+
+    /* backward solve from global1 into y */
+    ierr = VecSet(&zero,border); CHKERRQ(ierr);
+    ierr = VecPipelineBegin
+      (global2,border,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_DOWN,main_pipe);
+    CHKERRQ(ierr);
+    
+    ierr = VecPipelineIsStartOfPipe(main_pipe,PIPELINE_DOWN,&flg);
+    CHKERRQ(ierr);
+    if (!flg) {
+      ierr = MatMultAXBY_AIJ(mone,off_diag,border,one,x,global1);
+      CHKERRQ(ierr);
+    
       /* solve */
-      ierr = SLESSolve(local_method,tmp,tmq,&its); CHKERRQ(ierr);
-      /* undirect */
-      ierr = VecPlaceArray(tmp,p_ar); CHKERRQ(ierr);
-      ierr = VecPlaceArray(tmq,q_ar); CHKERRQ(ierr);
+      ierr = SLESSolve(local_method,local1,local2,&its); CHKERRQ(ierr);
     }
-    
     ierr = VecPipelineEnd
-      (global2,border, INSERT_VALUES, SCATTER_FORWARD,PIPELINE_UP,main_pipe);
-    CHKERRQ(ierr);
-
-    /* backward solve from global into y */
-    ierr = VecSet(&zero,border); CHKERRQ(ierr);
-    ierr = VecPipelineBegin
-      (y,border,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_DOWN,main_pipe);
-    CHKERRQ(ierr);
-    
-    ierr = MatMultAXBY_AIJ
-      (mone,off_diag,border,one,global,global2);
-    CHKERRQ(ierr);
-    
-    {/* some pointer rerouting to be able to solve into y */
-      Scalar *x_ar,*y_ar,*p_ar,*q_ar;
-
-      ierr = VecGetArray(tmp,&p_ar); CHKERRQ(ierr);
-      ierr = VecGetArray(global2,&x_ar); CHKERRQ(ierr);
-      ierr = VecPlaceArray(tmp,x_ar); CHKERRQ(ierr);
-
-      ierr = VecGetArray(tmq,&q_ar); CHKERRQ(ierr);
-      ierr = VecGetArray(y,&y_ar); CHKERRQ(ierr);
-      ierr = VecPlaceArray(tmq,y_ar); CHKERRQ(ierr);
-
-      ierr = SLESSolve(local_method,tmp,tmq,&its); CHKERRQ(ierr);
-
-      ierr = VecPlaceArray(tmp,p_ar); CHKERRQ(ierr);
-      ierr = VecPlaceArray(tmq,q_ar); CHKERRQ(ierr);
-    }
-    
-    ierr = VecPipelineEnd
-      (y,border,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_DOWN,main_pipe);
+      (global2,border,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_DOWN,main_pipe);
     CHKERRQ(ierr);
+    ierr = VecCopy(global2,y); CHKERRQ(ierr);
   }
   
   ierr = PetscPopErrorHandler(); CHKERRQ(ierr);
@@ -329,9 +334,8 @@ static int PCApplyrich_GBSSOR(PC pc,Vec b,Vec y,Vec w,int its)
 
 #undef __FUNC__
 #define __FUNC__ "PCDestroy_GBSSOR"
-int PCDestroy_GBSSOR(PetscObject obj)
+int PCDestroy_GBSSOR(PC pc)
 {
-  PC pc = (PC) obj;
   PC_GenBlockSSOR_struct *pc_data = (PC_GenBlockSSOR_struct *) pc->data;
   SLES local_method;
   int ierr;
@@ -368,16 +372,18 @@ static int PCSetFromOptions_GBSSOR(PC pc)
 
 #undef __FUNC__
 #define __FUNC__ "PCView_GBSSOR"
-static int PCView_GBSSOR(PetscObject obj,Viewer viewer)
+static int PCView_GBSSOR(PC pc,Viewer viewer)
 {
-  PC         pc = (PC)obj;
   PC_GenBlockSSOR_struct *pc_data = (PC_GenBlockSSOR_struct *) pc->data;
+  int ierr;
 
+  PetscFunctionBegin;
   if (pc_data->global_factorisation)
     PetscPrintf(pc->comm,"Global factorisation\n");
   else
     PetscPrintf(pc->comm,"No global factorisation\n");
-  return 0;
+  ierr = PCSubdomainsView(pc,viewer); CHKERRQ(ierr);
+  PetscFunctionReturn(0);
 }
 
 #undef __FUNC__
@@ -389,18 +395,21 @@ int PCCreate_GBSSOR(PC pc)
   pc->apply     = PCApply_GBSSOR;
   pc->applyrich = PCApplyrich_GBSSOR;
   pc->destroy   = PCDestroy_GBSSOR;
-  pc->setfrom   = PCSetFromOptions_GBSSOR;
+  pc->setfromoptions   = PCSetFromOptions_GBSSOR;
   pc->printhelp = 0;
   pc->setup     = PCSetup_GBSSOR;
-  pc->type      = PCGenBlockSSOR;
+  /*  pc->type      = PCGenBlockSSOR;*/
   pc->view      = PCView_GBSSOR;
 
   ierr = PCParallelSubdomainsCreate(pc,sizeof(PC_GenBlockSSOR_struct));
-
-  ((PC_GenBlockSSOR_struct *) pc->data)->global_factorisation = 0;
-
   ierr = PCParallelSubdomainPipelineSetType
-    (pc,PIPELINE_SEQUENTIAL); CHKERRQ(ierr);
+    (pc,PIPELINE_SEQUENTIAL,(PetscObject)PETSC_NULL); CHKERRQ(ierr);
+
+  {
+    PC_GenBlockSSOR_struct *pc_data = (PC_GenBlockSSOR_struct *) pc->data;
+    pc_data->global_factorisation = 0;
+    pc_data->fillmethod = FillDiag;
+  }
 
   return 0;
 }
