@@ -3,9 +3,9 @@
 */
 #include "src/vec/vecimpl.h"
 #include "src/mat/impls/aij/mpi/mpiaij.h"
+#include "parpre_pc.h"
 #include "parpre_subdomains.h"
 #include "pcschwarz.h"
-#include "src/pc/pcextra.h"
 
 #define CHUNCKSIZE   100
 
@@ -22,7 +22,7 @@ static int PCApply_AddSchwarz(PC pc,Vec x,Vec y)
   int ierr,its;
   Scalar zero = 0.0;
 
-  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,MPI_COMM_WORLD);
+  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,0);
   CHKERRQ(ierr);
   /*VecView(x,0);*/
   ierr = VecScatterBegin
@@ -56,6 +56,9 @@ static int PCApply_AddSchwarz(PC pc,Vec x,Vec y)
 
 }
 
+int ForceAddSchwarz()
+{return 0;}
+
 #undef __FUNC__
 #define __FUNC__ "PCCreate_AddSchwarz"
 int PCCreate_AddSchwarz(PC pc)
@@ -68,15 +71,14 @@ int PCCreate_AddSchwarz(PC pc)
   pc->printhelp = 0;
   pc->view      = PCView_Schwarz;
   pc->setup     = PCSetup_Schwarz;
-  pc->setfrom   = PCSetFromOptions_Schwarz;
-  pc->type      = PCAdditiveSchwarz;
+  pc->setfromoptions   = PCSetFromOptions_Schwarz;
 
   ierr = PCParallelSubdomainsCreate(pc,sizeof(PC_Schwarz_struct));
   CHKERRQ(ierr);
 
   ierr = PCSchwartzInstallSSORSubSolve(pc); CHKERRQ(ierr);
   ierr = PCParallelSubdomainPipelineSetType
-    (pc,PIPELINE_NONE); CHKERRQ(ierr);
+    (pc,PIPELINE_NONE,(PetscObject)PETSC_NULL); CHKERRQ(ierr);
 
   return 0;
 }
