@@ -3,8 +3,8 @@
 */
 #include "petsc.h"
 #include "src/vec/vecimpl.h"
-#include "src/mat/impls/aij/mpi/mpiaij.h"
-/* #include "../src/vec/utils/vpipe.h"*/
+/* #include "src/mat/impls/aij/mpi/mpiaij.h"*/
+#include "parpre_pc.h"
 #include "parpre_pipeline.h"
 #include "parpre_subdomains.h"
 #include "pcschwarz.h"
@@ -13,6 +13,7 @@
 
 extern int MatMultAXBY_AIJ(Scalar a, Mat aijin,Vec xx,
 			   Scalar b, Vec yy,Vec zz);
+extern int PCCustomPipelineSetFromOptions(PC pc);
 extern int PCSetFromOptions_Schwarz(PC pc);
 
 extern int ParPreTraceBackErrorHandler
@@ -35,9 +36,9 @@ static int PCApply_MulSchwarz(PC pc,Vec x,Vec y)
   int ierr,its;
   Scalar zero = 0.0;
 
-  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,MPI_COMM_WORLD);
+  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,0);
   CHKERRQ(ierr);
-  /*VecView(x,0);*/
+
   ierr = VecScatterBegin
     (x,pc_data->extended_vec,INSERT_VALUES,SCATTER_FORWARD,
      pc_data->scatter_to_extended);
@@ -53,18 +54,17 @@ static int PCApply_MulSchwarz(PC pc,Vec x,Vec y)
 		   pc_data->extended_vec,pc_data->extended_vec2,&its);
   /*  VecView(pc_data->extended_vec,0); VecView(pc_data->extended_vec2,0);*/
   CHKERRQ(ierr);
-
   VecSet(&zero,y);
 
   ierr = VecPipelineBegin
-    (pc_data->extended_vec2,y,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_DOWN,
+    (pc_data->extended_vec2,y,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_UP,
      pc_data->pipe_from_extended);
   CHKERRQ(ierr);
   ierr = VecPipelineEnd
-    (pc_data->extended_vec2,y,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_DOWN,
+    (pc_data->extended_vec2,y,INSERT_VALUES,SCATTER_FORWARD,PIPELINE_UP,
      pc_data->pipe_from_extended);
   CHKERRQ(ierr);
-  /*  VecView(y,0);*/
+
   ierr = PetscPopErrorHandler(); CHKERRQ(ierr);
 
   return 0;
@@ -95,15 +95,15 @@ int PCCreate_MulSchwarz(PC pc)
   pc->printhelp = 0;
   pc->view      = PCView_Schwarz;
   pc->setup     = PCSetup_Schwarz;
-  pc->setfrom   = PCSetFromOptions_MulSchwarz;
-  pc->type      = PCMultiplicativeSchwarz;
+  pc->setfromoptions   = PCSetFromOptions_MulSchwarz;
+  /*  pc->type      = PCMultiplicativeSchwarz;*/
 
   ierr = PCParallelSubdomainsCreate(pc,sizeof(PC_Schwarz_struct));
   CHKERRQ(ierr);
 
   ierr = PCSchwartzInstallSSORSubSolve(pc); CHKERRQ(ierr);
   ierr = PCParallelSubdomainPipelineSetType
-    (pc,PIPELINE_SEQUENTIAL); CHKERRQ(ierr);
+    (pc,PIPELINE_SEQUENTIAL,(PetscObject)PETSC_NULL); CHKERRQ(ierr);
 
   return 0;
 }
