@@ -17,8 +17,8 @@ typedef struct {
 
 extern int PCSchwarzSetHaloSize(PC pc, int ns);
 extern int PCSetFromOptions_Schwarz(PC pc);
-extern int PCDestroy_Schwarz(PetscObject obj);
+extern int PCDestroy_Schwarz(PC pc);
 extern int PCSchwarzGetLocalSLES(PC schwarz_pc,SLES *ssor_method);
 extern int PCSetup_Schwarz(PC pc);
-extern int PCView_Schwarz(PetscObject obj,Viewer viewer);
+extern int PCView_Schwarz(PC pc,Viewer viewer);
 extern int PCSchwartzInstallSSORSubSolve(PC pc);
