@@ -21,7 +21,7 @@ int PCSchwarzSetHaloSize(PC pc, int ns)
 {
   int ierr;
 
-  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,MPI_COMM_WORLD);
+  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,0);
   CHKERRQ(ierr);
   if (!PCIsSchwarzMethod(pc))
     SETERRQ(1,0,"Can only set halo for Schwarz methods");
@@ -36,6 +36,20 @@ int PCSchwarzSetHaloSize(PC pc, int ns)
   return 0;
 }
 
+#undef __FUNC__
+#define __FUNC__ "PCSchwarzGetIndices"
+int PCSchwarzGetIndices(PC pc,IS *is)
+{
+  PC_Schwarz_struct *pc_data = (PC_Schwarz_struct *) pc->data;
+
+  if (!PCIsSchwarzMethod(pc))
+    SETERRQ(1,0,"Trying to get Schwarz indices for non-Schwarz method");
+
+  *is = pc_data->Schwarz_rows;
+
+  return 0;
+}
+
 #undef __FUNC__
 #define __FUNC__ "PCSetFromOptions_Schwarz"
 int PCSetFromOptions_Schwarz(PC pc)
@@ -67,14 +81,15 @@ int PCSchwarzGetLocalSLES(PC schwarz_pc,SLES *local_sles)
 
 #undef __FUNC__
 #define __FUNC__ "PCSchwarzSetPipeType"
-static int PCSchwarzSetPipeType(PC schwarz_pc,PipelineType type)
+static int PCSchwarzSetPipeType(PC schwarz_pc,PipelineType type,PetscObject x)
 {
   PC_Schwarz_struct *pc_data = (PC_Schwarz_struct *) schwarz_pc->data;
   PC ssor_pc;
   int ierr;
 
   ierr = SLESGetPC(pc_data->ssor_solver,&ssor_pc); CHKERRQ(ierr);
-  ierr = PCParallelSubdomainPipelineSetType(ssor_pc,type); CHKERRQ(ierr);
+  ierr = PCParallelSubdomainPipelineSetType(ssor_pc,type,x);
+  CHKERRQ(ierr);
 
   return 0;
 }
@@ -131,6 +146,7 @@ static int GetOverlappingRange
   ierr = MatGetOwnershipRange(base_mat,&lo,&hi); CHKERRQ(ierr);
   domain_size = hi-lo;
   ierr = ISCreateStride(MPI_COMM_SELF,domain_size,lo,1,&domain); CHKERRQ(ierr);
+
   ierr =  MatIncreaseOverlap(base_mat,1,&domain,pc_data->halo_width);
   CHKERRQ(ierr);
   ierr = ISSort(domain); CHKERRQ(ierr); 
@@ -179,7 +195,7 @@ static int MakeSchwarzVectors(PC pc,int local_size,int *unwrap_low)
   CHKERRQ(ierr);
   ierr = PCParallelSubdomainPipelineGetType(pc,&pipe_type); CHKERRQ(ierr);
   ierr = VecPipelineSetType
-    (pc_data->pipe_from_extended,pipe_type,(PetscObject)(pc->mat));
+    (pc_data->pipe_from_extended,pipe_type,(PetscObject)(pc->pmat));
   CHKERRQ(ierr);
   ierr = VecPipelineSetup(pc_data->pipe_from_extended); CHKERRQ(ierr);
 
@@ -325,11 +341,11 @@ static int UnOverlapBlocks(Mat base_mat,PC pc)
 #define __FUNC__ "PCSetup_Schwarz"
 int PCSetup_Schwarz(PC pc)
 {
-  Mat base_mat = pc->mat;
+  Mat base_mat = pc->pmat;
   PC_Schwarz_struct *pc_data = (PC_Schwarz_struct *) pc->data;
   int local_size,ierr;
 
-  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,MPI_COMM_WORLD);
+  ierr = PetscPushErrorHandler(&ParPreTraceBackErrorHandler,0);
   CHKERRQ(ierr);
 
   if (!(base_mat->type==MATMPIAIJ)) {
@@ -347,7 +363,8 @@ int PCSetup_Schwarz(PC pc)
   ierr = UnOverlapBlocks(base_mat,pc); CHKERRQ(ierr);
     
   ierr = SLESSetOperators
-    (pc_data->ssor_solver,pc_data->Schwarz_mat,pc_data->Schwarz_mat,0);
+    (pc_data->ssor_solver,pc_data->Schwarz_mat,pc_data->Schwarz_mat,
+     (MatStructure)0);
   CHKERRQ(ierr);
   ierr = SLESSetUp(pc_data->ssor_solver,pc_data->extended_vec,
 		   pc_data->extended_vec2);
@@ -360,27 +377,28 @@ int PCSetup_Schwarz(PC pc)
 
 #undef __FUNC__
 #define __FUNC__ "PCView_Schwarz"
-int PCView_Schwarz(PetscObject obj,Viewer viewer)
+int PCView_Schwarz(PC pc,Viewer viewer)
 {
-  PC pc = (PC) obj;
   PC_Schwarz_struct *pc_data = (PC_Schwarz_struct *) pc->data;
+  PC ssor_pc;
   int ierr;
 
-  PetscPrintf(pc->comm,">> Schwarz subdomains\n");
-  ierr = PCSubdomainsView(pc,viewer); CHKERRQ(ierr);
+  PetscFunctionBegin;
   PetscPrintf(pc->comm," Halo width: %d\n",pc_data->halo_width);
-
-  return 0;
+  PetscPrintf(pc->comm,">> Schwarz subdomains\n");
+  ierr = SLESGetPC(pc_data->ssor_solver,&ssor_pc); CHKERRQ(ierr);
+  ierr = PCSubdomainsView(ssor_pc,viewer); CHKERRQ(ierr);
+  PetscFunctionReturn(0);
 }
 
 #undef __FUNC__
 #define __FUNC__ "PCDestroy_Schwarz"
-int PCDestroy_Schwarz(PetscObject obj)
+int PCDestroy_Schwarz(PC pc)
 {
-  PC pc = (PC) obj;
   PC_Schwarz_struct *pc_data = (PC_Schwarz_struct *) pc->data;
   int ierr;
 
+  PetscFunctionBegin;
   ierr = SLESDestroy(pc_data->ssor_solver); CHKERRQ(ierr);
   ierr = PCParallelDestroySubdomains(pc); CHKERRQ(ierr);
 
@@ -395,5 +413,5 @@ int PCDestroy_Schwarz(PetscObject obj)
 
   ierr = ISDestroy(pc_data->Schwarz_rows); CHKERRQ(ierr);
 
-  return 0;
+  PetscFunctionReturn(0);
 }
