@@ -11,15 +11,11 @@
 
 
 /* VE This should accomodate all parallel preconditioners */
-typedef struct {
-  SLES local_method;
-  CustomPipelineType pipe_type;
-  VecPipeline main_pipe;
-/*  PetscObject custom_pipe_data;*/
-  PipelineFunction up_fun,dn_fun;
-  int (*pipeline_setup)(VecPipeline,PetscObject*);
-  int (*pipeline_destroy)(PetscObject);
-} PCPstruct ;
+extern int VecPipelineSetCustomPipelineFromPCPstruct
+(VecPipeline,PCParallelStruct*);
+
+#endif
+
 /* ???
 struct _PCParallel_PipeInfo {
   CustomPipelineType pipe_type;
@@ -28,6 +24,3 @@ struct _PCParallel_PipeInfo {
 } ;
 */
 
-extern int VecPipelineSetCustomPipelineFromPCPstruct(VecPipeline,PCPstruct*);
-
-#endif
