@@ -6,9 +6,11 @@
 struct _PCParallelSubdomainStruct {
   SLES local_method;
   int (*getlocalmethod)(PC,SLES*);
-  int (*pipeline_settype)(PC,PipelineType);
+  int (*pipeline_settype)(PC,PipelineType,PetscObject);
   int (*pipeline_gettype)(PC,PipelineType*);
+  int (*pipeline_getobject)(PC,PetscObject*);
   VecPipeline main_pipe;
   PipelineType pipe_type; /* duplicated inside the main_pipe */
+  PetscObject pipe_obj;
 } ;
 
