@@ -1,14 +1,12 @@
+#include <string.h>
 #include "sys.h"
-#include "src/is/isimpl.h"
-#include "src/vec/vecimpl.h"                     /*I "vec.h" I*/
-#include "src/vec/impls/dvecimpl.h"
-#include "src/vec/impls/mpi/pvecimpl.h"
+#include "src/vec/impls/mpi/pvecimpl.h" /* definition of VecScatter_MPI_Gen */
 #include "src/mat/matimpl.h"
-#include "src/mat/impls/aij/mpi/mpiaij.h"
-#include "parpre_pipeline.h"
-#include "parpre_subdomains.h"
-#include "pipeline_impl.h"
 #include "src/pc/pcimpl.h"
+#include "src/mat/impls/aij/mpi/mpiaij.h"
+#include "parpre_subdomains.h"
+#include "parpre_pipeline.h"
+#include "pipeline_impl.h"
 #include "pinclude/pviewer.h"
 
 #undef __FUNC__
@@ -179,8 +177,36 @@ static int ProcColourUp(int proc,PetscObject pipe_info);
 static int ProcColourDown(int proc,PetscObject pipe_info);
 static int PipelineRedblackSetup(VecPipeline,PetscObject,PetscObject*);
 static int PipelineMulticolourSetup(VecPipeline,PetscObject,PetscObject*);
+static int ProcUserUp(int proc,PetscObject pipe_info);
+static int ProcUserDown(int proc,PetscObject pipe_info);
+static int PipelineUserSetup(VecPipeline,PetscObject,PetscObject*);
 #undef __FUNC__
 #define __FUNC__ "VecPipelineSetType"
+/*@
+   VecPipelineSetType - Sets the type of a vector pipeline. Vector
+   pipelines are to be used as
+
+   VecPipelineBegin(<see below for parameters>)
+   <do useful work with incoming data>
+   VecPipelineEnd(<see below for paramteres>)
+
+   Input Parameters:
++  ctx - vector pipeline object
++  type - vector pipeline type
+   Choices currently allowed are 
+   -- PIPELINE_NONE all processors send and receive simultaneously
+   -- PIPELINE_SEQUENTIAL processors send and receive in ascending
+   order of MPI rank
+   -- PIPELINE_RED_BLACK even numbered processors only send; odd numbered
+   processors only receive
+   -- PIPELINE_MULTICOLOUR processors are given a colour and send/receive
+   according to ascending colour
++  x - auxiliary data; for PIPELINE_MULTICOLOUR this should be
+   <(PetscObject) pmat> where pmat is the matrix on which the colouring
+   is to be based.
+
+.seealso: VecPipelineCreate, VecPipelineBegin, VecPipelineEnd.
+@*/
 int VecPipelineSetType(VecPipeline ctx,PipelineType type,PetscObject x)
 {
   ctx->pipe_type = type;
@@ -201,18 +227,23 @@ int VecPipelineSetType(VecPipeline ctx,PipelineType type,PetscObject x)
     ctx->upfn = &ProcColourUp;
     ctx->dnfn = &ProcColourDown;
     ctx->setup = &PipelineMulticolourSetup;
+  } else if (type == PIPELINE_USER) {
+    ctx->upfn = &ProcUserUp;
+    ctx->dnfn = &ProcUserDown;
+    ctx->setup = &PipelineUserSetup;
   } else SETERRQ(1,(int)type,
 		 "VecPipelineSetType: unknown or not implemented type\n");
 
   return 0;
 }
 
-/*@
-   VecPipelineBegin - Begins a generalized pipeline from one vector to
-   another. Complete the pipelineing phase with VecPipelineEnd().
-@*/
 #undef __FUNC__
 #define __FUNC__ "VecPipelineBegin"
+/*@
+   VecPipelineBegin - Receive data from processor earlier in
+   a processor pipeline from one vector to another. 
+.seealso: VecPipelineEnd.
+@*/
 int VecPipelineBegin(Vec x,Vec y,InsertMode addv,
 		     ScatterMode smode,PipelineDirection pmode,VecPipeline ctx)
 {
@@ -242,7 +273,9 @@ int VecPipelineBegin(Vec x,Vec y,InsertMode addv,
     else {
       gen_to   = (VecScatter_MPI_General *) scat->todata;
     }
+    /* force posting and waiting-for of receives only */
     if (ctx->pipe_type != PIPELINE_NONE) {
+      /* if no pipe, we do a regular scatter */
       nsends   = gen_to->n;
       gen_to->n = 0;
     }
@@ -253,19 +286,20 @@ int VecPipelineBegin(Vec x,Vec y,InsertMode addv,
   return 0;
 }
 
-/* --------------------------------------------------------------------*/
-/*@
-   VecPipelineEnd - Ends a generalized pipeline from one vector to another.  Call
-   after first calling VecPipelineBegin().
-@*/
 #undef __FUNC__
 #define __FUNC__ "VecPipelineEnd"
+/*@
+   VecPipelineEnd - Send data to processors later in
+   a processor pipeline from one vector to another.
+ 
+.seealso: VecPipelineBegin.
+@*/
 int VecPipelineEnd(Vec x,Vec y,InsertMode addv,
 		   ScatterMode smode,PipelineDirection pmode, VecPipeline ctx)
 {
   VecScatter scat = ctx->scatter;
   VecScatter_MPI_General *gen_from,*gen_to;
-  int nsends,nrecvs,ierr;
+  int nsends,nrecvs,local_n,ierr;
   
   if (smode & SCATTER_REVERSE ){
     gen_to   = (VecScatter_MPI_General *) scat->fromdata;
@@ -275,26 +309,56 @@ int VecPipelineEnd(Vec x,Vec y,InsertMode addv,
     gen_to   = (VecScatter_MPI_General *) scat->todata;
     gen_from = (VecScatter_MPI_General *) scat->fromdata;
   }
+  /* force posting and waiting-for of sends only,
+     prevent local copies, because we've alread done that in pipelinebegin */
+  nrecvs   = gen_from->n; gen_from->n = 0;
+  local_n  = gen_to->local.n; gen_to->local.n = 0;
   if (ctx->pipe_type == PIPELINE_NONE) {
-      nsends   = gen_to->n;
-      gen_to->n = 0;
+    /* if no pipe, we are already finished */
+    nsends   = gen_to->n;
+    gen_to->n = 0;
   }
-  nrecvs   = gen_from->n;
-  gen_from->n = 0;
   ierr = VecScatterBegin(x,y,addv,smode,scat); CHKERRQ(ierr);
   ierr = VecScatterEnd(x,y,addv,smode,scat); CHKERRQ(ierr);
   if (ctx->pipe_type == PIPELINE_NONE) gen_to->n = nsends;
-  gen_from->n = nrecvs;
+  gen_from->n = nrecvs; gen_to->local.n = local_n;
 
   return 0;
 }
 
+#undef __FUNC__
+#define __FUNC__ "VecPipelineIsStartOfPipe"
+/*@
+   VecPipelineIsStartOfPipe - Test whether this processor has no predecessors.
+@*/
+int VecPipelineIsStartOfPipe(VecPipeline ctx,PipelineDirection pmode,int *flg)
+{
+  VecScatter_MPI_General *gen_from;
+  int ierr;
+
+  PetscFunctionBegin;
+  if (!ctx->setupcalled) {
+    ierr = VecPipelineSetup(ctx); CHKERRQ(ierr);
+  }
+
+  if (pmode==PIPELINE_UP) {
+    gen_from = (VecScatter_MPI_General *) ctx->upfrom;
+  } else if (pmode==PIPELINE_DOWN) {
+    gen_from = (VecScatter_MPI_General *) ctx->dnfrom;
+  } else SETERRQ(1,pmode,
+		 "VecPipelineIsStartOfPipe: unknown or not implemented pipeline mode");
+
+  *flg = gen_from->n==0;
+
+  PetscFunctionReturn(0);
+}
+
+#undef __FUNC__
+#define __FUNC__ "VecPipelineDestroy"
 /*@C
    VecPipelineDestroy - Destroys a pipeline context created by 
    VecPipelineCreate().
 @*/
-#undef __FUNC__
-#define __FUNC__ "VecPipelineDestroy"
 int VecPipelineDestroy( VecPipeline ctx )
 {
   int ierr;
@@ -304,7 +368,7 @@ int VecPipelineDestroy( VecPipeline ctx )
   return 0;
 }
 
-/* >>>> Routines for sequential ordering of processors <<<< */
+/* Pipeline setup: sequential */
 
 typedef struct {int mytid;} Pipeline_sequential_info;
 
@@ -339,6 +403,8 @@ static int ProcUp(int proc,PetscObject pipe_info)
     return 0;
   }
 }
+#undef __FUNC__
+#define __FUNC__ "ProcDown"
 static int ProcDown(int proc,PetscObject pipe_info)
 { 
   int mytid = ((Pipeline_sequential_info *)pipe_info)->mytid;
@@ -357,19 +423,22 @@ static int PipelineSequentialSetup(VecPipeline vs,PetscObject x,
 {
   Pipeline_sequential_info *info;
 
+  PetscFunctionBegin;
   info = PetscNew(Pipeline_sequential_info);
   MPI_Comm_rank(vs->scatter->comm,&(info->mytid));
   *obj = (PetscObject) info;
 
-  return 0;
+  PetscFunctionReturn(0);
 }
 
-/* >>>> Routines for multicolour ordering of processors <<<< */
+/* Pipeline setup: multicolour */
 
 typedef struct {
   int mytid,numtids,*proc_colours;
 } Pipeline_coloured_info;
 
+#undef __FUNC__
+#define __FUNC__ "ProcColourUp"
 static int ProcColourUp(int proc,PetscObject pipe_info)
 {
   Pipeline_coloured_info* comm_info =
@@ -382,6 +451,8 @@ static int ProcColourUp(int proc,PetscObject pipe_info)
   else
     return 0;
 }
+#undef __FUNC__
+#define __FUNC__ "ProcColourDown"
 static int ProcColourDown(int proc,PetscObject pipe_info)
 { 
   Pipeline_coloured_info* comm_info =
@@ -406,7 +477,7 @@ static int PipelineRedblackSetup(VecPipeline vs,PetscObject x,
   info = PetscNew(Pipeline_coloured_info);
   MPI_Comm_rank(vs->scatter->comm,&(info->mytid));
   MPI_Comm_size(vs->scatter->comm,&numtids);
-  info->proc_colours = PetscMalloc(numtids*sizeof(int));
+  info->proc_colours = (int *) PetscMalloc(numtids*sizeof(int));
   CHKPTRQ(info->proc_colours);
   for (i=0; i<numtids; i++) {info->proc_colours[i] = i%2;}
   *obj = (PetscObject) info;
@@ -426,7 +497,7 @@ static int PipelineMulticolourSetup(VecPipeline vs,PetscObject x,
   info = PetscNew(Pipeline_coloured_info);
   MPI_Comm_rank(mat->comm,&(info->mytid));
   MPI_Comm_size(mat->comm,&numtids);
-  info->proc_colours = PetscMalloc(numtids*sizeof(int));
+  info->proc_colours = (int *) PetscMalloc(numtids*sizeof(int));
   CHKPTRQ(info->proc_colours);
   PetscMemzero(info->proc_colours,numtids*sizeof(int));
 
@@ -497,6 +568,67 @@ static int PipelineMulticolourSetup(VecPipeline vs,PetscObject x,
   return 0;
 }
 
+/* Pipeline setup: from user list */
+
+typedef struct {
+  int mytid,numtids,*ordering;
+} Pipeline_user_info;
+
+#undef __FUNC__
+#define __FUNC__ "ProcUserUp"
+static int ProcUserUp(int proc,PetscObject pipe_info)
+{
+  Pipeline_user_info *comm_info = (Pipeline_user_info *) pipe_info;
+  int myloc,*ordering = comm_info->ordering,i;
+
+  PetscFunctionBegin;
+  myloc = 1;
+  for (i=1; i<=ordering[0]; i++)
+    if (ordering[i]==comm_info->mytid) {
+      myloc = i; break; }
+  for (i=ordering[0]; i>0; i--) {
+    if (ordering[i]==proc) PetscFunctionReturn(1);
+    if (i==myloc)          PetscFunctionReturn(0);
+  }
+  SETERRQ(1,0,"User pipeline ordering did not include my tid\n");
+}
+#undef __FUNC__
+#define __FUNC__ "ProcUserDown"
+static int ProcUserDown(int proc,PetscObject pipe_info)
+{
+  Pipeline_user_info *comm_info = (Pipeline_user_info *) pipe_info;
+  int myloc,*ordering,i;
+
+  PetscFunctionBegin;
+  ordering = comm_info->ordering;
+  myloc = ordering[0]-1;
+  for (i=ordering[0]; i>0; i--)
+    if (ordering[i]==comm_info->mytid) {
+      myloc = i; break; }
+  for (i=1; i<=ordering[0]; i++) {
+    if (ordering[i]==proc) PetscFunctionReturn(1);
+    if (i==myloc)          PetscFunctionReturn(0);
+  }
+  SETERRQ(1,0,"User pipeline ordering did not include my tid\n");
+}
+#undef __FUNC__
+#define __FUNC__ "PipelineUserSetup"
+static int PipelineUserSetup(VecPipeline vs,PetscObject x,
+			     PetscObject *obj)
+{
+  Pipeline_user_info *info;
+
+  PetscFunctionBegin;
+  info = PetscNew(Pipeline_user_info);
+  MPI_Comm_rank(vs->scatter->comm,&(info->mytid));
+  MPI_Comm_size(vs->scatter->comm,&(info->numtids));
+  info->ordering = (int *) x;
+  *obj = (PetscObject) info;
+
+  PetscFunctionReturn(0);
+}
+
+
 #undef __FUNC__
 #define __FUNC__ "PCCustomPipelineSetFromOptions"
 int PCCustomPipelineSetFromOptions(PC pc)
@@ -510,16 +642,20 @@ int PCCustomPipelineSetFromOptions(PC pc)
   if (flag) {
     value[4]='\0';
     if (strcmp(value,"none")==0) {
-      ierr = PCParallelSubdomainPipelineSetType(pc,PIPELINE_NONE);
+      ierr = PCParallelSubdomainPipelineSetType
+	(pc,PIPELINE_NONE,(PetscObject)PETSC_NULL);
       CHKERRQ(ierr);
     } else if (strcmp(value,"sequ")==0) {
-      ierr = PCParallelSubdomainPipelineSetType(pc,PIPELINE_SEQUENTIAL);
+      ierr = PCParallelSubdomainPipelineSetType
+	(pc,PIPELINE_SEQUENTIAL,(PetscObject)PETSC_NULL);
       CHKERRQ(ierr);
     } else if (strcmp(value,"redb")==0) {
-      ierr = PCParallelSubdomainPipelineSetType(pc,PIPELINE_REDBLACK);
+      ierr = PCParallelSubdomainPipelineSetType
+	(pc,PIPELINE_REDBLACK,(PetscObject)PETSC_NULL);
       CHKERRQ(ierr);
     } else if (strcmp(value,"mult")==0) {
-      ierr = PCParallelSubdomainPipelineSetType(pc,PIPELINE_MULTICOLOUR);
+      ierr = PCParallelSubdomainPipelineSetType
+	(pc,PIPELINE_MULTICOLOUR,(PetscObject)pc->pmat);
       CHKERRQ(ierr);
     } else SETERRQ(1,0,"unknown custom pipeline option");
   }
@@ -532,24 +668,20 @@ int VecPipelineView(VecPipeline pipe,Viewer viewer)
 {
   MPI_Comm comm = pipe->comm;
 
-  PetscPrintf(comm,">> Vector Pipeline<<\n");
+  PetscPrintf(comm,"Vector Pipeline:\n");
   if (!pipe->setupcalled) PetscPrintf(comm,"Not yet set up\n");
-  PetscPrintf(comm,"Pipelinetype: %d\n",(int)pipe->pipe_type);
-  PetscPrintf(comm,"based on scatter:\n");
-  /*  ierr = VecScatterView(pipe->scatter,viewer); CHKERRQ(ierr);*/
-  PetscPrintf(comm,"Up function %d\n",(int)pipe->upfn);
-  PetscPrintf(comm,"Dn function %d\n",(int)pipe->dnfn);
+  PetscPrintf(comm,"Pipelinetype: ");
+  switch ((int)pipe->pipe_type) {
+  case PIPELINE_NONE: PetscPrintf(comm,"none\n"); break;
+  case PIPELINE_SEQUENTIAL: PetscPrintf(comm,"sequential\n"); break;
+  case PIPELINE_REDBLACK: PetscPrintf(comm,"red-black\n"); break;
+  case PIPELINE_MULTICOLOUR: PetscPrintf(comm,"multicolour\n"); break;
+  case PIPELINE_USER: PetscPrintf(comm,"user defined\n"); break;
+  }
+  /*PetscPrintf(comm,"based on scatter:\n");
+    ierr = VecScatterView(pipe->scatter,viewer); CHKERRQ(ierr);
+    PetscPrintf(comm,"Up function %d\n",(int)pipe->upfn);
+    PetscPrintf(comm,"Dn function %d\n",(int)pipe->dnfn);*/
 
   return 0;
 }
-
-#undef __FUNC__
-#define __FUNC__ "VecScatterCopyToPipeline_PtoP"
-int VecScatterCopyToPipeline_PtoP(VecScatter in,VecPipeline *out)
-{
-
-printf("\n    Calling disabled routine VecScatterCopyToPipeline\n\n");
-
-  return 0;
-}
-
