@@ -8,7 +8,7 @@ typedef struct _PCParallel_PipeInfo *PCParallel_PipeInfo;
 typedef int (*PipelineFunction)(int,PetscObject);
 
 struct _VecPipeline {
-  PETSCHEADER
+  PETSCHEADER(int)
   VecScatter scatter;
   PipelineType pipe_type; /* duplicated in the subdomain structure */
   VecScatter_MPI_General *upto,*upfrom,*dnto,*dnfrom;
